﻿using StudiApp.Services.Converters;

namespace StudiApp.Test.Unit.ServicesTests.ConverterTests;

[TestFixture]
public class PercentageConverterTest
{
	[TestCase(0.5, "50")]
	[TestCase(0.99, "99")]
	[TestCase(0.01, "1")]
	[TestCase(0.75, "75")]
	public void ToStringTest(double percentage, string expected)
	{
		Assert.That(PercentageConverter.ConvertToString(percentage), Is.EqualTo(expected));
	}

	[TestCase("50", 0.5)]
	[TestCase("99", 0.99)]
	[TestCase("1", 0.01)]
	[TestCase("75", 0.75)]
	public void ToDoubleTest(string percentage, double expected)
	{
		Assert.That(PercentageConverter.ConvertToDouble(percentage), Is.EqualTo(expected));
	}

	[TestCase(1.1)]
	[TestCase(0.0)]
	public void InvalidDoubleValueTest(double value)
	{
		Assert.That(() => PercentageConverter.ConvertToString(value),
			Throws.TypeOf<ArgumentException>().With.Message
				.EqualTo("The Percentage is over 100% (1.0) or under 1% (0.01)"));
	}


	[TestCase("0")]
	[TestCase("110")]
	public void InvalidStringValueTest(string value)
	{
		Assert.That(() => PercentageConverter.ConvertToDouble(value),
			Throws.TypeOf<ArgumentException>().With.Message
				.EqualTo("The Percentage is over 100% (1.0) or under 1% (0.01)"));
	}
}