using StudiApp.Models;
using StudiApp.Services.Facade;

namespace StudiApp.Test.Unit.ServicesTests.FacadeTests;

public class TimetableItemFacadeTests
{
	private TimetableItemFacade _timetableItemFacade = null!;

	[SetUp]
	public void Setup()
	{
		_timetableItemFacade = new TimetableItemFacade();
	}

	[Test]
	public void TestFromTodos()
	{
		// Arrange
		var todo = new ToDo
		{
			Id = 25,
			Name = "TestTodo",
			Description = "TestTodo",
			Due = new DateTime(2023, 05, 06)
		};

		var todos = new List<ToDo> {todo};

		// Act
		var timetableItems = _timetableItemFacade.FromTodos(todos, "");

		// Assert
		Assert.That(timetableItems, Has.Count.EqualTo(1));
		Assert.Multiple(() =>
		{
			Assert.That(timetableItems[0].Id, Is.EqualTo($"{todo.Id}-"));
			Assert.That(timetableItems[0].Name, Is.EqualTo(todo.Name));
		});
	}

	[Test]
	[Category("SyncfusionBehavior")]
	public void TestFromLecturesWithCourse()
	{
		// Arrange
		var lecture = new Lecture
		{
			Id = 87,
			Name = "TestLectureABC$$\\",
			DayTime = new DayTime
			{
				DayOfWeek = DayOfWeek.Monday,
				StartTime = new TimeOnly(08, 10),
				EndTime = new TimeOnly(08, 55)
			},
			Course = new Course
			{
				Id = 4,
				Semester = new Semester
				{
					Id = 32,
					DateRange = new DateRange
					{
						Start = new DateTime(2023, 03, 01),
						End = new DateTime(2023, 07, 31)
					}
				}
			}
		};

		var lectures = new List<Lecture> {lecture};

		// Act
		var timetableItems = _timetableItemFacade.FromLectures(lectures, "");

		// Assert
		Assert.That(timetableItems, Has.Count.EqualTo(1));
		Assert.Multiple(() =>
		{
			Assert.That(timetableItems[0].Id, Is.EqualTo($"{lecture.Course.Id}-"));
			Assert.That(timetableItems[0].Name, Is.EqualTo(lecture.Name));
			Assert.That(timetableItems[0].StartTime, Is.EqualTo(new DateTime(2023, 03, 06, 8, 10, 0)));
			Assert.That(timetableItems[0].EndTime, Is.EqualTo(new DateTime(2023, 03, 06, 8, 55, 0)));
		});
	}
}