﻿using Moq;
using StudiApp.ViewModels;
using StudiApp.Views;

namespace StudiApp.Test.Unit.ViewModelsTests.CourseModule;

[TestFixture]
public class CourseDetailViewModelTest
{
	[SetUp]
	public void SetUp()
	{
		_helper = new CourseModuleTestHelper();
		_vm = new CoursesDetailViewModel(_helper.ViewModelService.Object)
		{
			Id = 1
		};
	}

	private CoursesDetailViewModel _vm = null!;
	private CourseModuleTestHelper _helper = null!;

	[Test]
	public void AppearingCommandTest()
	{
		_vm.AppearingCommand.Execute(null);

		//Assertions
		_helper.RepositoryManager.Verify(r => r.GetCourse(_vm.Id), Times.Once);

		Assert.Multiple(() =>
		{
			Assert.That(_vm.Course.Name, Is.EqualTo("ActiveCourse"));
			Assert.That(_vm.Course.Id, Is.EqualTo(1));
			Assert.That(_vm.Lectures, Has.Count.EqualTo(2));
			Assert.That(_vm.Lectures.First().Name, Is.EqualTo("Lecture1"));
			Assert.That(_vm.Lectures.Last().Name, Is.EqualTo("Lecture2"));
			Assert.That(_vm.ToDos.First().Name, Is.EqualTo("ToDo1"));
			Assert.That(_vm.ToDos.Last().Name, Is.EqualTo("ToDo2"));
			Assert.That(_vm.AverageMark, Is.EqualTo(4.0));
		});
	}

	[Test]
	public void CompleteCommandTest()
	{
		_vm.AppearingCommand.Execute(null);
		_vm.ToggleCompleteCommand.Execute(_helper.ToDo1);

		//Assertions
		_helper.RepositoryManager.Verify(r => r.SaveCourse(_helper.CourseActive));
		Assert.Multiple(() =>
		{
			Assert.That(_helper.ToDo1.IsCompleted, Is.True);
			Assert.That(_vm.ToDos, Has.Count.EqualTo(1));
		});
	}

	[Test]
	public void EditCommandTest()
	{
		_vm.EditCommand.Execute(_helper.CourseActive);

		//Assertions
		_helper.NavigationManager.Verify(n => n.NavigateTo(nameof(CoursesEditView), "Id", _helper.CourseActive.Id),
			Times.Once);
	}
}