﻿using Moq;
using StudiApp.ViewModels;

namespace StudiApp.Test.Unit.ViewModelsTests.CourseModule;

[TestFixture]
public class CourseNewViewModelTest
{
	[SetUp]
	public void SetUp()
	{
		_helper = new CourseModuleTestHelper();
		_vm = new CoursesNewViewModel(_helper.ViewModelService.Object);
	}

	private CoursesNewViewModel _vm = null!;
	private CourseModuleTestHelper _helper = null!;

	[Test]
	public void AppearingCommandTest()
	{
		_vm.AppearingCommand.Execute(null);

		//Assertions
		_helper.RepositoryManager.Verify(r => r.GetAllSemesters(null), Times.Once);

		Assert.Multiple(() =>
		{
			//Assert Empty Course
			Assert.That(_vm.Course.Name, Is.EqualTo(null));
			Assert.That(_vm.Course.LecturerName, Is.EqualTo(null));
			Assert.That(_vm.Course.Semester, Is.EqualTo(null));
			Assert.That(_vm.Course.Description, Is.EqualTo(null));
			Assert.That(_vm.Course.Ects, Is.EqualTo(0));
			Assert.That(_vm.Course.Marks, Has.Count.EqualTo(0));
			Assert.That(_vm.Course.Lectures, Has.Count.EqualTo(0));
			Assert.That(_vm.Course.ToDos, Has.Count.EqualTo(0));

			//Assert SelectedSemester
			Assert.That(_vm.SelectedSemester, Is.EqualTo(null));

			//Assert Lectures
			Assert.That(_vm.Lectures, Has.Count.EqualTo(1));
			Assert.That(_vm.Lectures.First().DayTime.DayOfWeek, Is.EqualTo(DayOfWeek.Monday));
			Assert.That(_vm.Lectures.First().Name, Is.EqualTo(null));

			//Assert WeekDays
			Assert.That(_vm.WeekDays[0], Is.EqualTo(DayOfWeek.Sunday));
			Assert.That(_vm.WeekDays[1], Is.EqualTo(DayOfWeek.Monday));
			Assert.That(_vm.WeekDays[2], Is.EqualTo(DayOfWeek.Tuesday));
			Assert.That(_vm.WeekDays[3], Is.EqualTo(DayOfWeek.Wednesday));
			Assert.That(_vm.WeekDays[4], Is.EqualTo(DayOfWeek.Thursday));
			Assert.That(_vm.WeekDays[5], Is.EqualTo(DayOfWeek.Friday));
			Assert.That(_vm.WeekDays[6], Is.EqualTo(DayOfWeek.Saturday));

			//Assert Semesters
			Assert.That(_vm.Semesters.First().Name, Is.EqualTo("Inactive"));
			Assert.That(_vm.Semesters.First().Id, Is.EqualTo(1));
			Assert.That(_vm.Semesters.Last().Name, Is.EqualTo("Active"));
			Assert.That(_vm.Semesters.Last().Id, Is.EqualTo(2));
		});
	}
}