﻿using Moq;
using StudiApp.Models;
using StudiApp.ViewModels;
using StudiApp.Views;

namespace StudiApp.Test.Unit.ViewModelsTests.CourseModule;

[TestFixture]
public class CoursesOverviewViewModelTest
{
	[SetUp]
	public void SetUp()
	{
		_helper = new CourseModuleTestHelper();
		_vm = new CoursesOverviewViewModel(_helper.ViewModelService.Object);
	}

	private CoursesOverviewViewModel _vm = null!;
	private CourseModuleTestHelper _helper = null!;

	[Test]
	public void AppearingCommandTest()
	{
		_vm.AppearingCommand.Execute(null);

		//Assertions
		_helper.RepositoryManager.Verify(r => r.GetAllCourses(null), Times.Once);
		_helper.RepositoryManager.Verify(r => r.GetAllSemesters(null), Times.Once);

		Assert.Multiple(() =>
		{
			Assert.That(_vm.Title, Is.EqualTo("Active Courses"));
			Assert.That(_vm.Courses, Has.Count.EqualTo(2));
			Assert.That(_vm.ShownCourses, Has.Count.EqualTo(1));
			Assert.That(_vm.ShownCourses.First().Name, Is.EqualTo("ActiveCourse"));
		});
	}

	[Test]
	public void NewCourseCommandTest()
	{
		_vm.NewCourseClickedCommand.Execute(null);

		//Assertions
		_helper.NavigationManager.Verify(n => n.NavigateTo(nameof(CoursesNewView)), Times.Once);
	}

	[Test]
	public void TapCommandTest()
	{
		_vm.TapCommand.Execute(_helper.CourseActive);

		//Assertions
		_helper.NavigationManager.Verify(n => n.NavigateTo(nameof(CoursesDetailView), "Id", _helper.CourseActive.Id),
			Times.Once);
	}

	[Test]
	public void ShowAllCommandTest()
	{
		_vm.AppearingCommand.Execute(null);
		_vm.IsShowAllSet = true;
		_vm.ShowAllClickedCommand.Execute(null);

		//Assertions
		Assert.Multiple(() =>
		{
			Assert.That(_vm.Title, Is.EqualTo("All Courses"));
			Assert.That(_vm.ShownCourses, Has.Count.EqualTo(2));
			Assert.That(_vm.ShownCourses.First().Name, Is.EqualTo("ActiveCourse"));
			Assert.That(_vm.ShownCourses.Last().Name, Is.EqualTo("NotActiveCourse"));
		});
	}

	[Test]
	public void DeleteCommandTest()
	{
		var course = new Course
		{
			Id = 1,
			Name = "Course1"
		};

		_vm.DeleteCourseCommand.Execute(course);

		//Assertions
		_helper.DialogService.Verify(
			d => d.ShowDeleteConfirmationAsync("Course", course.Name),
			Times.Once);
		_helper.RepositoryManager.Verify(r => r.DeleteCourse(course), Times.Once);
	}
}