﻿using Moq;
using StudiApp.Models;
using StudiApp.ViewModels;
using StudiApp.Views;

namespace StudiApp.Test.Unit.ViewModelsTests.DashboardModule;

[TestFixture]
public class DashboardViewModelTest
{
	[SetUp]
	public void SetUp()
	{
		_helper = new DashboardModuleTestHelper();
		_vm = new DashboardViewModel(_helper.ViewModelService.Object);
	}

	private DashboardModuleTestHelper _helper = null!;
	private DashboardViewModel _vm = null!;

	[Test]
	public void AppearingCommandTest_WithSemester()
	{
		_vm.AppearingCommand.Execute(null);

		//Assertions
		_helper.RepositoryManager.Verify(r => r.GetAllSemesters(null), Times.Once);
		_helper.RepositoryManager.Verify(r => r.GetAllToDos(null), Times.Once);

		Assert.Multiple(() =>
		{
			Assert.That(_vm.ToDos, Has.Count.EqualTo(1));
			Assert.That(_vm.ToDos.First().Name, Is.EqualTo("TodoInNextUp"));
			Assert.That(_vm.Courses, Has.Count.EqualTo(2));
			Assert.That(_vm.Courses.First().Name, Is.EqualTo("CourseActive"));
			Assert.That(_vm.DoubleCourseItems, Has.Count.EqualTo(1));
			Assert.That(_vm.DoubleCourseItems.First().FirstCourse.Name, Is.EqualTo("CourseActive"));
			Assert.That(_vm.DoubleCourseItems.First().SecondCourse.Name, Is.EqualTo("SecondCourseActive"));
			Assert.That(_vm.Lectures, Has.Count.EqualTo(1));
			Assert.That(_vm.Lectures.First().Name, Is.EqualTo("LectureInNextUp"));
			Assert.That(_vm.NextUpItems, Has.Count.EqualTo(2));
		});
	}

	[Test]
	public void AppearingCommandTest_WithoutSemester()
	{
		//Update Mocks
		_helper.RepositoryManager.Setup(r => r.GetAllSemesters(null)).Returns(Task.FromResult(new List<Semester>()));
		_helper.ViewModelService.Setup(v => v.RepositoryManager).Returns(_helper.RepositoryManager.Object);

		_vm.AppearingCommand.Execute(null);

		//Assertions
		_helper.RepositoryManager.Verify(r => r.GetAllSemesters(null), Times.Once);
		_helper.RepositoryManager.Verify(r => r.GetAllToDos(null), Times.Once);


		Assert.Multiple(() =>
		{
			Assert.That(_vm.ToDos, Has.Count.EqualTo(1));
			Assert.That(_vm.ToDos.First().Name, Is.EqualTo("TodoInNextUp"));
			Assert.That(_vm.Courses, Is.Null);
			Assert.That(_vm.DoubleCourseItems, Is.Empty);
			Assert.That(_vm.Lectures, Has.Count.EqualTo(0));
			Assert.That(_vm.NextUpItems, Has.Count.EqualTo(1));
		});
	}

	[Test]
	public void TapCourseCommandTest()
	{
		_vm.TapCourseCommand.Execute(_helper.CourseActive);

		//Assertions
		_helper.NavigationManager.Verify(n => n.NavigateTo(nameof(CoursesDetailView), "Id", _helper.CourseActive.Id),
			Times.Once);
	}

	[Test]
	public void TapNextUpItemCommandTest()
	{
		//Input
		_vm.TapNextUpItemCommand.Execute(_helper.LectureNextUpItem);

		//Assertion
		_helper.NavigationManager.Verify(
			n => n.NavigateTo(nameof(CoursesDetailView), "Id", _helper.LectureNextUpItem.Id), Times.Once);

		//Input
		_vm.TapNextUpItemCommand.Execute(_helper.TodoNextUpItem);

		//Assertion
		_helper.NavigationManager.Verify(n => n.NavigateTo(nameof(ToDoEditView), "Id", _helper.TodoNextUpItem.Id),
			Times.Once);
	}
}