﻿using Moq;
using StudiApp.ViewModels;

namespace StudiApp.Test.Unit.ViewModelsTests.MarkModule;

[TestFixture]
public class MarkEditViewModelTest
{
	[SetUp]
	public void SetUp()
	{
		_helper = new MarkModuleTestHelper();
		_vm = new MarksEditViewModel(_helper.ViewModelService)
		{
			Id = _helper.Mark1.Id,
			SelectedSemester = _helper.SemesterActive
		};
	}

	private MarksEditViewModel _vm;
	private MarkModuleTestHelper _helper;

	[Test]
	public void AppearingCommandTest()
	{
		_vm.AppearingCommand.Execute(null);

		Assert.Multiple(() =>
		{
			var mark = _helper.Mark1;
			Assert.That(_vm.Mark.Id, Is.EqualTo(mark.Id));
			Assert.That(_vm.Mark.Value, Is.EqualTo(mark.Value));
			Assert.That(_vm.Mark.Percentage, Is.EqualTo(mark.Percentage));
			Assert.That(_vm.Courses, Is.EqualTo(_helper.SemesterActive.Courses));
		});
	}

	[Test]
	public void SaveCommandValidTest()
	{
		_vm.AppearingCommand.Execute(null);
		_vm.SaveCommand.Execute(null);

		_helper.RepositoryManager.Verify(r => r.SaveMark(_helper.Mark1), Times.Once);
		_helper.NavigationManager.Verify(n => n.NavigateBack(), Times.Once);
	}

	[Test]
	public void SaveCommandInvalidTest()
	{
		_vm.AppearingCommand.Execute(null);
		_vm.Mark.Value = -1;
		_vm.SaveCommand.Execute(null);

		_helper.DialogService.Verify(d => d.ShowAlertAsync(It.IsAny<string>()), Times.Once);
		_helper.RepositoryManager.Verify(r => r.SaveMark(_helper.Mark1), Times.Never);
		_helper.NavigationManager.Verify(n => n.NavigateBack(), Times.Never);
	}

	[Test]
	public void SaveCommandInvalidPercentageTest()
	{
		_vm.AppearingCommand.Execute(null);
		_vm.Percentage = string.Empty;
		_vm.SaveCommand.Execute(null);

		_helper.DialogService.Verify(d => d.ShowAlertAsync(It.IsAny<string>()), Times.Once);
		_helper.RepositoryManager.Verify(r => r.SaveMark(_helper.Mark1), Times.Never);
		_helper.NavigationManager.Verify(n => n.NavigateBack(), Times.Never);
	}
}