﻿using Microsoft.Extensions.Logging;
using Moq;
using StudiApp.Interfaces;
using StudiApp.Models;
using StudiApp.Services.InputValidator;
using StudiApp.Services.ViewModelService;

namespace StudiApp.Test.Unit.ViewModelsTests.MarkModule;

public class MarkModuleTestHelper
{
	internal Course CourseActive;
	internal Course CourseActive2;
	internal Course CourseNotActive;
	internal Mock<IDialogService> DialogService;
	internal IInputValidator InputValidator;
	internal Mock<ILogger<ViewModelService>> Logger;
	internal Mark Mark1;
	internal Mark Mark2;
	internal Mark Mark3;
	internal Mock<INavigationManager> NavigationManager;
	internal Mock<IRepositoryManager> RepositoryManager;
	internal Semester SemesterActive;
	internal Semester SemesterNotActive;
	internal IViewModelService ViewModelService;

	internal MarkModuleTestHelper()
	{
		SemesterNotActive = new Semester
		{
			Name = "Inactive",
			DateRange = new DateRange
			{
				Start = new DateTime(1999, 01, 01),
				End = new DateTime(2000, 01, 01)
			},
			Courses = new List<Course> {CourseNotActive!},
			Id = 1
		};
		SemesterActive = new Semester
		{
			Name = "Active",
			DateRange = new DateRange
			{
				Start = DateTime.Now.AddDays(-100),
				End = DateTime.Now.AddDays(100)
			},
			Courses = new List<Course> {CourseActive!, CourseActive2!},
			Id = 2
		};
		var allSemesters = new List<Semester> {SemesterNotActive, SemesterActive};


		Mark1 = new Mark
		{
			Id = 1,
			Value = 3.0,
			Percentage = 0.5
		};
		Mark2 = new Mark
		{
			Id = 2,
			Value = 5.0,
			Percentage = 0.5
		};
		Mark3 = new Mark
		{
			Id = 3,
			Value = 5.5,
			Percentage = 0.5
		};
		var allMarks = new List<Mark> {Mark1, Mark2, Mark3};


		CourseActive = new Course
		{
			Name = "ActiveCourse",
			Semester = SemesterActive,
			SemesterId = 2,
			LecturerName = "LecturerName",
			Description = "Description",
			Ects = 4,
			Id = 1,
			IsPassed = true,
			Marks = new List<Mark> {Mark1, Mark2}
		};
		Mark1.Course = CourseActive;
		CourseActive2 = new Course
		{
			Name = "ActiveCourse2",
			Semester = SemesterActive,
			SemesterId = 2,
			LecturerName = "LecturerName",
			Description = "Description",
			Ects = 4,
			Id = 2,
			IsPassed = false,
			Marks = new List<Mark>()
		};
		CourseNotActive = new Course
		{
			Name = "NotActiveCourse",
			Semester = SemesterNotActive,
			SemesterId = 1,
			LecturerName = "LecturerName",
			Description = "Description",
			Ects = 2,
			Id = 3,
			IsPassed = true,
			Marks = new List<Mark> {Mark3}
		};
		var allCourses = new List<Course> {CourseActive, CourseActive2, CourseNotActive};


		DialogService = new Mock<IDialogService>();
		NavigationManager = new Mock<INavigationManager>();
		RepositoryManager = new Mock<IRepositoryManager>();
		Logger = new Mock<ILogger<ViewModelService>>();
		InputValidator = new InputValidator();


		RepositoryManager.Setup(r => r.GetAllSemesters(null)).Returns(Task.FromResult(allSemesters));
		RepositoryManager.Setup(r => r.GetAllCourses(null)).Returns(Task.FromResult(allCourses));
		RepositoryManager.Setup(r => r.GetAllMarks(null)).Returns(Task.FromResult(allMarks));
		RepositoryManager.Setup(r => r.DeleteMark(Mark1)).Returns(() =>
		{
			allMarks.Remove(Mark1);
			CourseActive.Marks.Remove(Mark1);
			return Task.FromResult(true);
		});
		RepositoryManager.Setup(r => r.GetMark(Mark1.Id)).Returns(Task.FromResult(Mark1));

		DialogService.Setup(d => d.ShowDeleteConfirmationAsync(
				It.IsAny<string>(),
				It.IsAny<string>()))
			.Returns(Task.FromResult(true));

		ViewModelService = new ViewModelService(DialogService.Object,
			NavigationManager.Object,
			RepositoryManager.Object,
			Logger.Object,
			InputValidator);
	}
}