﻿using Moq;
using StudiApp.ViewModels;
using StudiApp.Views;

namespace StudiApp.Test.Unit.ViewModelsTests.MarkModule;

[TestFixture]
public class MarkOverviewViewModelTest
{
	[SetUp]
	public void SetUp()
	{
		_helper = new MarkModuleTestHelper();
		_vm = new MarksOverviewViewModel(_helper.ViewModelService);
	}

	private MarksOverviewViewModel _vm;
	private MarkModuleTestHelper _helper;

	[Test]
	public void AppearingCommandTest()
	{
		_vm.AppearingCommand.Execute(null);

		_helper.RepositoryManager.Verify(r => r.GetAllMarks(null), Times.Once);
		_helper.RepositoryManager.Verify(r => r.GetAllSemesters(null), Times.Once);
		_helper.RepositoryManager.Verify(r => r.GetAllCourses(null), Times.Once);

		Assert.Multiple(() =>
		{
			Assert.That(_vm.SelectedSemester, Is.EqualTo(_helper.SemesterActive));
			Assert.That(_vm.Marks, Is.EqualTo(_helper.CourseActive.Marks));
			Assert.That(_vm.SemesterEcts, Is.EqualTo(4));
			Assert.That(_vm.SemesterAverageMark, Is.EqualTo(4));
			Assert.That(_vm.TotalEcts, Is.EqualTo(6));
			Assert.That(_vm.TotalAverageMark, Is.EqualTo(4.5));
		});
	}

	[Test]
	public void NewMarkCommandTest()
	{
		_vm.NewMarkCommand.Execute(null);

		_helper.NavigationManager
			.Verify(n => n.NavigateTo(nameof(MarksCreateView),
				"SelectedSemester",
				_vm.SelectedSemester));
	}

	[Test]
	public void EditMarkCommandTest()
	{
		_vm.EditMarkCommand.Execute(_helper.Mark1);

		_helper.NavigationManager
			.Verify(n => n.NavigateTo(nameof(MarksEditView), new Dictionary<string, object>
			{
				{"Id", _helper.Mark1.Id},
				{"SelectedSemester", _vm.SelectedSemester}
			}));
	}

	[Test]
	public void DeleteMarkCommandTest()
	{
		_vm.AppearingCommand.Execute(null);
		_vm.DeleteMarkCommand.Execute(_helper.Mark1);

		_helper.DialogService.Verify(d => d.ShowDeleteConfirmationAsync("Mark", ""));
		_helper.RepositoryManager.Verify(r => r.DeleteMark(_helper.Mark1), Times.Once);

		Assert.Multiple(() =>
		{
			Assert.That(_vm.Marks, Has.Count.EqualTo(1));
			var mark = _vm.Marks.First();
			Assert.That(mark.Id, Is.EqualTo(2));
			Assert.That(mark.Value, Is.EqualTo(5));
			Assert.That(mark.Percentage, Is.EqualTo(0.5));
		});
	}
}