﻿using Moq;
using StudiApp.ViewModels;

namespace StudiApp.Test.Unit.ViewModelsTests.SettingsModule;

internal class SemestersEditViewModelTest
{
	private SettingsModuleTestHelper _helper = null!;
	private SemestersEditViewModel _vm = null!;

	[SetUp]
	public void SetUp()
	{
		_helper = new SettingsModuleTestHelper();
		_vm = new SemestersEditViewModel(_helper.ViewModelService) {Id = 1};
	}

	[Test]
	public void AppearingCommandTest()
	{
		_vm.AppearingCommand.Execute(null);

		_helper.RepositoryManager.Verify(r => r.GetSemester(_vm.Id), Times.Once);

		Assert.Multiple(() =>
		{
			Assert.That(_vm.Semester.Id, Is.EqualTo(_helper.Semester1.Id));
			Assert.That(_vm.Semester.Name, Is.EqualTo(_helper.Semester1.Name));
			Assert.That(_vm.Semester.DateRange, Is.EqualTo(_helper.Semester1.DateRange));
		});
	}

	[Test]
	public void SaveCommandValidTest()
	{
		_vm.AppearingCommand.Execute(null);
		_vm.SaveCommand.Execute(null);

		_helper.RepositoryManager.Verify(r => r.SaveSemester(_vm.Semester), Times.Once);
		_helper.NavigationManager.Verify(n => n.NavigateBack(), Times.Once);
	}

	[Test]
	public void SaveCommandNotValidTest()
	{
		_vm.AppearingCommand.Execute(null);
		_vm.Semester.Name = "";
		_vm.SaveCommand.Execute(null);

		_helper.RepositoryManager.Verify(r => r.SaveSemester(_vm.Semester), Times.Never);
		_helper.NavigationManager.Verify(n => n.NavigateBack(), Times.Never);
	}
}