﻿using Microsoft.Extensions.Logging;
using Moq;
using StudiApp.Interfaces;
using StudiApp.Models;
using StudiApp.Services.InputValidator;
using StudiApp.Services.ViewModelService;

namespace StudiApp.Test.Unit.ViewModelsTests.SettingsModule;

internal class SettingsModuleTestHelper
{
	internal List<Semester> allSemesters;
	internal Mock<IDialogService> DialogService;
	internal IInputValidator InputValidator;
	internal Mock<ILogger<ViewModelService>> Logger;
	internal Mock<INavigationManager> NavigationManager;
	internal Mock<IRepositoryManager> RepositoryManager;
	internal Semester Semester1;
	internal Semester Semester2;
	internal ViewModelService ViewModelService;

	internal SettingsModuleTestHelper()
	{
		var today = DateTime.Today;
		Semester1 = new Semester
		{
			Name = "Semester 1",
			DateRange = new DateRange
			{
				Start = today,
				End = today.AddMonths(6)
			},
			Id = 1
		};

		Semester2 = new Semester
		{
			Name = "Semester 2",
			DateRange = new DateRange
			{
				Start = today.AddMonths(-6),
				End = today.AddDays(-1)
			},
			Id = 2
		};

		allSemesters = new List<Semester> {Semester1, Semester2};

		DialogService = new Mock<IDialogService>();
		NavigationManager = new Mock<INavigationManager>();
		RepositoryManager = new Mock<IRepositoryManager>();
		Logger = new Mock<ILogger<ViewModelService>>();
		InputValidator = new InputValidator();

		RepositoryManager
			.Setup(r => r.GetAllSemesters(null))
			.Returns(Task.FromResult(allSemesters));
		RepositoryManager
			.Setup(r => r.GetSemester(Semester1.Id))
			.Returns(Task.FromResult(Semester1));

		DialogService.Setup(d => d.ShowDeleteConfirmationAsync(
				It.IsAny<string>(),
				It.IsAny<string>()))
			.Returns(Task.FromResult(true));

		ViewModelService = new ViewModelService(DialogService.Object,
			NavigationManager.Object,
			RepositoryManager.Object,
			Logger.Object,
			InputValidator);
	}
}