﻿using Moq;
using StudiApp.ViewModels;
using StudiApp.Views;

namespace StudiApp.Test.Unit.ViewModelsTests.SettingsModule;

[TestFixture]
public class SettingsViewModelTest
{
	[SetUp]
	public void SetUp()
	{
		_helper = new SettingsModuleTestHelper();
		_vm = new SettingsViewModel(_helper.ViewModelService);
	}

	private SettingsViewModel _vm = null!;
	private SettingsModuleTestHelper _helper = null!;

	[Test]
	public void AppearingCommandTest()
	{
		_vm.AppearingCommand.Execute(null);

		_helper.RepositoryManager.Verify(r => r.GetAllSemesters(null), Times.Once);
		Assert.That(_vm.Semesters, Has.Count.EqualTo(2));
	}

	[Test]
	public void NewSemesterCommandTest()
	{
		_vm.NewSemesterCommand.Execute(null);

		_helper.NavigationManager.Verify(n => n.NavigateTo(nameof(SemestersCreateView)), Times.Once);
	}

	[Test]
	public void EditSemesterCommandTest()
	{
		_vm.EditSemesterCommand.Execute(_helper.Semester1);

		_helper.NavigationManager.Verify(n => n.NavigateTo(nameof(SemestersEditView), "Id", _helper.Semester1.Id),
			Times.Once);
	}

	[Test]
	public async Task DeleteSemesterCommandTest()
	{
		_vm.AppearingCommand.Execute(null);
		_helper.allSemesters.Remove(_helper.Semester1);
		await _vm.DeleteSemester(_helper.Semester1);

		_helper.DialogService.Verify(d => d.ShowDeleteConfirmationAsync("Semester", _helper.Semester1.Name),
			Times.Once);

		Assert.Multiple(() =>
		{
			Assert.That(_vm.Semesters, Has.Count.EqualTo(1));
			var semester = _vm.Semesters.First();
			Assert.That(semester.Id, Is.EqualTo(_helper.Semester2.Id));
			Assert.That(semester.Name, Is.EqualTo(_helper.Semester2.Name));
			Assert.That(semester.DateRange, Is.EqualTo(_helper.Semester2.DateRange));
		});
	}
}