﻿using System.Linq.Expressions;
using Microsoft.Extensions.Logging;
using Moq;
using StudiApp.Interfaces;
using StudiApp.Models;
using StudiApp.Services.ViewModelService;

namespace StudiApp.Test.Unit.ViewModelsTests.ToDoModule;

internal class ToDoModuleTestHelper
{
	internal readonly Course Course1;
	internal readonly Course Course2;

	internal readonly ToDo Todo1;
	internal readonly ToDo Todo2;

	internal bool ShowArchived = false;

	internal ToDoModuleTestHelper()
	{
		Course1 = new Course
		{
			Id = 1,
			Name = "Course1",
			Description = "Description1",
			Ects = 2,
			IsPassed = false,
			LecturerName = "Lecturer1",
			Lectures = new List<Lecture>
			{
				new()
				{
					Name = "Lecture1",
					Room = "1.111"
				}
			}
		};

		Course2 = new Course
		{
			Id = 2,
			Name = "Course2",
			Description = "Description2",
			Ects = 4,
			IsPassed = true,
			LecturerName = "Lecturer2",
			Lectures = new List<Lecture>
			{
				new()
				{
					Name = "Lecture2",
					Room = "1.112"
				}
			}
		};

		Todo1 = new ToDo
		{
			Id = 1,
			Course = Course1,
			CourseId = Course1.Id,
			Name = "ToDo1",
			Description = "Description1",
			Due = new DateTime(2023, 05, 12),
			IsCompleted = false
		};

		Todo2 = new ToDo
		{
			Id = 2,
			Course = Course2,
			CourseId = Course2.Id,
			Name = "ToDo2",
			IsCompleted = true
		};

		ViewModelServiceMock = new Mock<IViewModelService>();
		RepositoryManagerMock = new Mock<IRepositoryManager>();
		NavigationManagerMock = new Mock<INavigationManager>();
		DialogServiceMock = new Mock<IDialogService>();
		InputValidatorMock = new Mock<IInputValidator>();
		LoggerMock = new Mock<ILogger<ViewModelService>>();

		ViewModelServiceMock.Setup(p => p.RepositoryManager).Returns(RepositoryManagerMock.Object);
		ViewModelServiceMock.Setup(p => p.NavigationManager).Returns(NavigationManagerMock.Object);
		ViewModelServiceMock.Setup(p => p.InputValidator).Returns(InputValidatorMock.Object);
		ViewModelServiceMock.Setup(p => p.DialogService).Returns(DialogServiceMock.Object);
		ViewModelServiceMock.Setup(p => p.Logger).Returns(LoggerMock.Object);

		RepositoryManagerMock.Setup(rm => rm.GetAllCourses(null)).Returns(GetCourses);
		RepositoryManagerMock.Setup(rm => rm.GetAllToDos(It.IsAny<Expression<Func<ToDo, bool>>>()))
			.Returns(GetToDos);
	}

	internal Mock<IViewModelService> ViewModelServiceMock { get; }
	internal Mock<IRepositoryManager> RepositoryManagerMock { get; }
	internal Mock<INavigationManager> NavigationManagerMock { get; }
	internal Mock<IInputValidator> InputValidatorMock { get; }
	internal Mock<IDialogService> DialogServiceMock { get; }
	internal Mock<ILogger<ViewModelService>> LoggerMock { get; }

	internal IViewModelService ViewModelService => ViewModelServiceMock.Object;

	internal Task<List<Course>> GetCourses()
	{
		return Task.FromResult(new List<Course>
		{
			Course1, Course2
		});
	}

	internal Task<List<ToDo>> GetToDos()
	{
		return Task.FromResult(new List<ToDo>
		{
			Todo1, Todo2
		}.Where(t => t.IsCompleted == ShowArchived).ToList());
	}
}