﻿using CommunityToolkit.Maui;
using MetroLog.MicrosoftExtensions;
using MetroLog.Operators;
using StudiApp.Interfaces;
using StudiApp.Models;
using StudiApp.Services.Dialog;
using StudiApp.Services.Facade;
using StudiApp.Services.InputValidator;
using StudiApp.Services.NavigationManager;
using StudiApp.Services.Repository;
using StudiApp.Services.ViewModelService;
using StudiApp.ViewModels;
using StudiApp.Views;
using Syncfusion.Licensing;
using Syncfusion.Maui.Core.Hosting;

namespace StudiApp;

public static class MauiProgram
{
	private const string SyncfusionLicenseKey =
		"Mgo+DSMBaFt+QHFqVkNrXVNbdV5dVGpAd0N3RGlcdlR1fUUmHVdTRHRcQlljQX9SdkVgXHtWd3U=;Mgo+DSMBPh8sVXJ1S0d+X1RPd11dXmJWd1p/THNYflR1fV9DaUwxOX1dQl9gSXpTfkRiWX9ddXFVRmE=;ORg4AjUWIQA/Gnt2VFhhQlJBfV5AQmBIYVp/TGpJfl96cVxMZVVBJAtUQF1hSn5Xdk1iWH9ec3FQR2Rd;MTY1OTI2N0AzMjMxMmUzMTJlMzMzNUlrTkU5S2lmR0EreFFXM0ZFNEZ0c2poOHdmSmNsR25Ick8wemp5VEhCL1E9;MTY1OTI2OEAzMjMxMmUzMTJlMzMzNWZFVG5UMEdrQlpXT0lUd0hIOGJQNVlEN1QxR2FKb0J1RWlHdk9Wa0liajg9;NRAiBiAaIQQuGjN/V0d+XU9Hc1RDX3xKf0x/TGpQb19xflBPallYVBYiSV9jS31TckVrWH5ecHdQQ2dYUA==;MTY1OTI3MEAzMjMxMmUzMTJlMzMzNVFkZ3B0VDR1OWxHSE01WGprWGZ3MG56SUZhSTdwTkNzNm14RkFkc1RBazQ9;MTY1OTI3MUAzMjMxMmUzMTJlMzMzNWRHY1JOMWZuMFNtUU9IT3FFejJJejZBMzd1S2NLKzk2YXplbXVNeVFXNFE9;Mgo+DSMBMAY9C3t2VFhhQlJBfV5AQmBIYVp/TGpJfl96cVxMZVVBJAtUQF1hSn5Xdk1iWH9ec3FTRWZd;MTY1OTI3M0AzMjMxMmUzMTJlMzMzNVBab3l5a0lDYkEzRUEwcDNqMURrNWdLSGttd001c0gxUlZIb2hUdjNCVDQ9;MTY1OTI3NEAzMjMxMmUzMTJlMzMzNUtzdnlWUmVCdzJnYkpkUk5kWUxjMmk3RlZybEI3NHpZbW93Q2hjQlpTdHM9;MTY1OTI3NUAzMjMxMmUzMTJlMzMzNVFkZ3B0VDR1OWxHSE01WGprWGZ3MG56SUZhSTdwTkNzNm14RkFkc1RBazQ9";

	public static MauiApp CreateMauiApp()
	{
		var builder = MauiApp.CreateBuilder();
		builder
			.UseMauiApp<App>()
			.UseMauiCommunityToolkit()
			.ConfigureSyncfusionCore()
			.ConfigureFonts(fonts =>
			{
				fonts.AddFont("OpenSans-Regular.ttf", "OpenSansRegular");
				fonts.AddFont("OpenSans-Semibold.ttf", "OpenSansSemibold");
			})
			.RegisterAppServices()
			.RegisterViewModels()
			.RegisterViews()
			.RegisterDatabase()
			.RegisterFacades()
			.RegisterLogger();

		// Key should not be hardcoded but Syncfusion says it's best practice (https://www.syncfusion.com/forums/177811/storing-licensing-key-best-practices)
		SyncfusionLicenseProvider.RegisterLicense(SyncfusionLicenseKey);

		return builder.Build();
	}

	private static void RegisterLogger(this MauiAppBuilder mauiAppBuilder)
	{
		mauiAppBuilder.Logging.AddTraceLogger(_ => { });

		mauiAppBuilder.Logging.AddStreamingFileLogger(options =>
		{
			const string loggingFolder = "Logs";
			var path = Path.Combine(FileSystem.AppDataDirectory, loggingFolder);

			options.RetainDays = 60;
			options.FolderPath = path;
		});

		mauiAppBuilder.Services.AddSingleton(LogOperatorRetriever.Instance);
	}

	public static MauiAppBuilder RegisterAppServices(this MauiAppBuilder mauiAppBuilder)
	{
		mauiAppBuilder.Services.AddSingleton<IInputValidator, InputValidator>();
		mauiAppBuilder.Services.AddSingleton<IDialogService, DialogService>();
		mauiAppBuilder.Services.AddSingleton<INavigationManager, NavigationManager>();
		mauiAppBuilder.Services.AddSingleton<IViewModelService, ViewModelService>();
		return mauiAppBuilder;
	}

	public static MauiAppBuilder RegisterViewModels(this MauiAppBuilder mauiAppBuilder)
	{
		mauiAppBuilder.Services.AddSingleton<DashboardViewModel>();
		mauiAppBuilder.Services.AddSingleton<MarksOverviewViewModel>();
		mauiAppBuilder.Services.AddSingleton<CoursesOverviewViewModel>();
		mauiAppBuilder.Services.AddSingleton<TimetableViewModel>();
		mauiAppBuilder.Services.AddSingleton<ToDoOverviewViewModel>();
		mauiAppBuilder.Services.AddSingleton<SettingsViewModel>();

		mauiAppBuilder.Services.AddTransient<CoursesEditViewModel>();
		mauiAppBuilder.Services.AddTransient<CoursesNewViewModel>();
		mauiAppBuilder.Services.AddTransient<CoursesDetailViewModel>();
		mauiAppBuilder.Services.AddTransient<ToDoEditViewModel>();
		mauiAppBuilder.Services.AddTransient<ToDoNewViewModel>();
		mauiAppBuilder.Services.AddTransient<MarksEditViewModel>();
		mauiAppBuilder.Services.AddTransient<MarksCreateViewModel>();
		mauiAppBuilder.Services.AddTransient<SemestersEditViewModel>();
		mauiAppBuilder.Services.AddTransient<SemestersCreateViewModel>();

		return mauiAppBuilder;
	}

	public static MauiAppBuilder RegisterViews(this MauiAppBuilder mauiAppBuilder)
	{
		mauiAppBuilder.Services.AddSingleton<DashboardView>();
		mauiAppBuilder.Services.AddSingleton<MarksOverviewView>();
		mauiAppBuilder.Services.AddSingleton<TimetableView>();
		mauiAppBuilder.Services.AddSingleton<CoursesOverviewView>();
		mauiAppBuilder.Services.AddSingleton<ToDoOverviewView>();
		mauiAppBuilder.Services.AddSingleton<SettingsView>();

		mauiAppBuilder.Services.AddTransient<CoursesEditView>();
		mauiAppBuilder.Services.AddTransient<CoursesNewView>();
		mauiAppBuilder.Services.AddTransient<CoursesDetailView>();
		mauiAppBuilder.Services.AddTransient<ToDoEditView>();
		mauiAppBuilder.Services.AddTransient<ToDoNewView>();
		mauiAppBuilder.Services.AddTransient<MarksEditView>();
		mauiAppBuilder.Services.AddTransient<MarksCreateView>();
		mauiAppBuilder.Services.AddTransient<SemestersCreateView>();
		mauiAppBuilder.Services.AddTransient<SemestersEditView>();

		return mauiAppBuilder;
	}

	public static MauiAppBuilder RegisterDatabase(this MauiAppBuilder mauiAppBuilder)
	{
		mauiAppBuilder.Services.AddSingleton<IRepositoryManager, RecursiveRepositoryManager>();

		mauiAppBuilder.Services.AddSingleton<IRepository<Mark, int>, SqliteRepository<Mark>>();
		mauiAppBuilder.Services.AddSingleton<IRepository<Lecture, int>, SqliteRepository<Lecture>>();
		mauiAppBuilder.Services.AddSingleton<IRepository<Course, int>, SqliteRepository<Course>>();
		mauiAppBuilder.Services.AddSingleton<IRepository<ToDo, int>, SqliteRepository<ToDo>>();
		mauiAppBuilder.Services.AddSingleton<IRepository<Semester, int>, SqliteRepository<Semester>>();

		return mauiAppBuilder;
	}

	public static MauiAppBuilder RegisterFacades(this MauiAppBuilder mauiAppBuilder)
	{
		mauiAppBuilder.Services.AddSingleton<ITimetableItemFacade, TimetableItemFacade>();

		return mauiAppBuilder;
	}
}