﻿using StudiApp.Models;

namespace StudiApp.Services.Converters;

public static class AverageMarkConverter
{
	private const string MarksValueErrorMessage = "A Value in the marks list is over 6.0 or under 1.0";

	private const string MarksPercentageErrorMessage =
		"A Percentage in the marks list is over 100% (1.0) or under 1% (0.01)";

	private const double ValueMin = 1.0;
	private const double ValueMax = 6.0;
	private const double PercentageMin = 0.01;
	private const double PercentageMax = 1.0;
	private const double DefaultValue = 0;

	public static double MarksToAverage(List<Mark> marks)
	{
		if (marks is not {Count: > 0}) return DefaultValue;

		if (marks.Any(m => m.Value is > ValueMax or < ValueMin))
			throw new ArgumentException(MarksValueErrorMessage);

		if (marks.Any(m => m.Percentage is > PercentageMax or < PercentageMin))
			throw new ArgumentException(MarksPercentageErrorMessage);

		var marksSum = marks.Sum(mark => mark.Value * mark.Percentage);
		var percentageSum = marks.Sum(mark => mark.Percentage);
		var average = marksSum / percentageSum;
		return Math.Round(average, 2);
	}
}