﻿namespace StudiApp.Services.Converters;

public static class PercentageConverter
{
	private const string PercentageErrorMessage = "The Percentage is over 100% (1.0) or under 1% (0.01)";
	private const double PercentageMin = 0.01;
	private const double PercentageMax = 1.0;
	private const double PercentageStringMin = 1;
	private const double PercentageStringMax = 100;
	private const double PercentageModifier = 100;

	public static string ConvertToString(double value)
	{
		if (value is > PercentageMax or < PercentageMin)
			throw new ArgumentException(PercentageErrorMessage);

		var percent = value * PercentageModifier;
		return $"{(int) percent}";
	}

	public static double ConvertToDouble(string value)
	{
		if (!double.TryParse(value, out var doubleRepresentation))
			doubleRepresentation = 0.0;

		if (doubleRepresentation is > PercentageStringMax or < PercentageStringMin)
			throw new ArgumentException(PercentageErrorMessage);

		return doubleRepresentation / PercentageModifier;
	}
}