using StudiApp.Interfaces;
using StudiApp.Models;

namespace StudiApp.Services.Facade;

public class TimetableItemFacade : ITimetableItemFacade
{
	private const int WeekdayAbbreviationLength = 2;
	private const string SemesterFormatting = "yyyyMMddTHHmmssZ";
	private const int StartTimeShift = -20;

	public List<TimetableItem> FromLectures(List<Lecture> lectures, string destinationView)
	{
		var timetableItems = new List<TimetableItem>();
		foreach (var lecture in lectures)
		{
			var timetableItem = new TimetableItem
			{
				Id = $"{lecture.Course.Id}-{destinationView}",
				Name = lecture.Name
			};

			var semester = lecture.Course.Semester;
			var startTime = semester.DateRange.Start;

			timetableItem.StartTime = lecture.DayTime.GetStartTimeOfFirstOccurrence(startTime);
			timetableItem.EndTime = lecture.DayTime.GetEndTimeOfFirstOccurrence(startTime);
			timetableItem.RecurrenceRule = GetRecurrenceRule(lecture.DayTime.DayOfWeek, semester.DateRange.End);

			timetableItems.Add(timetableItem);
		}

		return timetableItems;
	}

	public List<TimetableItem> FromTodos(List<ToDo> todos, string destinationView)
	{
		var timetableItems = new List<TimetableItem>();
		foreach (var todo in todos)
		{
			var timetableItem = new TimetableItem
			{
				Id = $"{todo.Id}-{destinationView}",
				Name = todo.Name,
				StartTime = todo.Due.AddMinutes(StartTimeShift),
				EndTime = todo.Due
			};

			timetableItems.Add(timetableItem);
		}

		return timetableItems;
	}

	private static string GetRecurrenceRule(DayOfWeek day, DateTime end)
	{
		var semesterEnd = end.AddDays(1).ToString(SemesterFormatting); // AddDays to include last day
		var weekdayAbbreviation = day.ToString().Substring(0, WeekdayAbbreviationLength).ToUpper();

		return $"FREQ=WEEKLY;INTERVAL=1;UNTIL={semesterEnd};BYDAY={weekdayAbbreviation}";
	}
}