using System.Text;

namespace StudiApp.Services.InputValidator.Strategy;

public abstract class ValidatorStrategyBase
{
	protected const int NameStringMinValue = 1;
	protected const int NameStringMaxValue = 100;
	protected const int DescriptionStringMaxValue = 250;

	private static string ConcatenateErrorMessages(List<ValidatorItem> validatorItemList)
	{
		var stringBuilder = new StringBuilder();
		foreach (var item in validatorItemList) stringBuilder.Append(item.ErrorMessage);
		return stringBuilder.ToString();
	}

	protected static string GetErrorMessageName(string property)
	{
		return $"{property} Name between {NameStringMinValue} - {NameStringMaxValue} chars\n";
	}

	protected static string GetErrorMessageDescription(string property)
	{
		return $"{property} Description can't be longer than {DescriptionStringMaxValue} chars\n";
	}

	protected static string GetErrorMessageTime(string property)
	{
		return $"{property} StartTime has to be before EndTime\n";
	}

	protected static string GetErrorMessageSelection(string property, string selection)
	{
		return $"Please choose a {selection} for the {property}\n";
	}

	public abstract ValidatorItem IsValid();

	public static ValidatorItem MergeValidatorItems(List<ValidatorItem> validatorItemList)
	{
		var validatorItem = new ValidatorItem(validatorItemList.All(item => item.IsValid));

		if (validatorItem.IsValid) return validatorItem;

		validatorItem.ErrorMessage = ConcatenateErrorMessages(validatorItemList);

		return validatorItem;
	}
}