using System.Linq.Expressions;
using SQLite;
using SQLiteNetExtensions.Extensions.TextBlob;
using SQLiteNetExtensionsAsync.Extensions;
using StudiApp.Interfaces;
using StudiApp.Services.Serializers;

namespace StudiApp.Services.Repository;

public class SqliteRepository<T> : IRepository<T, int>
	where T : class, IRepositoryItem<int>, new()
{
	private const string DatabaseFilename = "StudiApp.db3";

	private const SQLiteOpenFlags Flags =
		// open the database in read/write mode
		SQLiteOpenFlags.ReadWrite |
		// create the database if it doesn't exist
		SQLiteOpenFlags.Create |
		// enable multi-threaded database access
		SQLiteOpenFlags.SharedCache;

	private readonly SQLiteAsyncConnection _database;

	static SqliteRepository()
	{
		TextBlobOperations.SetTextSerializer(new DatabaseTextBlobSerializer());
	}

	public SqliteRepository()
	{
		_database = new SQLiteAsyncConnection(DatabasePath, Flags);
		var task = _database.CreateTableAsync<T>();
		task.Wait();
	}

	private static string DatabasePath =>
		Path.Combine(FileSystem.AppDataDirectory, DatabaseFilename);

	public async Task<T> Create(T item)
	{
		await _database.InsertWithChildrenAsync(item);
		return item;
	}

	public async Task<List<T>> GetAll(Expression<Func<T, bool>> query = null)
	{
		return await _database.GetAllWithChildrenAsync(query);
	}

	public async Task<T> Get(int id)
	{
		return await _database.FindWithChildrenAsync<T>(id);
	}

	public async Task<T> Update(T item)
	{
		await _database.UpdateWithChildrenAsync(item);
		return item;
	}

	public async Task<T> Delete(T item)
	{
		await _database.DeleteAsync(item);
		return item;
	}
}