﻿using System.Collections.ObjectModel;
using CommunityToolkit.Mvvm.ComponentModel;
using CommunityToolkit.Mvvm.Input;
using Microsoft.Extensions.Logging;
using StudiApp.Interfaces;
using StudiApp.Models;
using StudiApp.Services.Converters;
using StudiApp.Views;

namespace StudiApp.ViewModels;

[QueryProperty("Id", "Id")]
public partial class CoursesDetailViewModel : ObservableObject
{
	private readonly IViewModelService _viewModelService;

	[ObservableProperty] private double _averageMark;
	[ObservableProperty] private Course _course;
	[ObservableProperty] private ObservableCollection<Lecture> _lectures = new();
	[ObservableProperty] private ObservableCollection<ToDo> _toDos = new();

	public CoursesDetailViewModel(IViewModelService viewModelService)
	{
		_viewModelService = viewModelService;
	}

	public int Id { get; set; }

	private ObservableCollection<Lecture> GetLectures()
	{
		var lectures = new ObservableCollection<Lecture>();

		foreach (var lecture in Course.Lectures) lectures.Add(lecture);

		_viewModelService.Logger.LogInformation("Next Lectures loaded");
		return lectures;
	}

	private double GetAverageMark()
	{
		return AverageMarkConverter.MarksToAverage(Course.Marks);
	}

	private ObservableCollection<ToDo> GetToDos()
	{
		var toDos = new ObservableCollection<ToDo>();

		foreach (var todo in Course.ToDos.Where(t => !t.IsCompleted)) toDos.Add(todo);

		_viewModelService.Logger.LogInformation("ToDos loaded");

		return toDos;
	}

	[RelayCommand]
	public async Task OnAppearing()
	{
		Course = await _viewModelService.RepositoryManager.GetCourse(Id);
		_viewModelService.Logger.LogInformation("Course {CourseName} loaded", Course.Name);

		Lectures = GetLectures();
		AverageMark = GetAverageMark();
		ToDos = GetToDos();

		_viewModelService.Logger.LogInformation("Detail Page for {CourseName} loaded", Course.Name);
	}

	[RelayCommand]
	public async Task Edit(Course c)
	{
		await _viewModelService.NavigationManager.NavigateTo(nameof(CoursesEditView), "Id", c.Id);
		_viewModelService.Logger.LogInformation("Navigating to {ViewName}", nameof(CoursesEditView));
	}

	[RelayCommand]
	public async Task ToggleComplete(ToDo todo)
	{
		todo.IsCompleted = true;
		await _viewModelService.RepositoryManager.SaveCourse(Course);
		ToDos = GetToDos();
		_viewModelService.Logger.LogInformation("ToDo ({ToDoName}) saved", todo.Name);
	}
}