﻿using CommunityToolkit.Mvvm.ComponentModel;
using CommunityToolkit.Mvvm.Input;
using Microsoft.Extensions.Logging;
using StudiApp.Interfaces;
using StudiApp.Models;
using StudiApp.Services.InputValidator.Strategy;

namespace StudiApp.ViewModels;

public abstract partial class SemestersViewModelBase : ObservableObject
{
	protected readonly IViewModelService _viewModelService;

	[ObservableProperty] protected Semester _semester;

	protected SemestersViewModelBase(IViewModelService viewModelService)
	{
		_viewModelService = viewModelService;
	}

	[RelayCommand]
	public async Task Save()
	{
		var validatorResult = _viewModelService.InputValidator.IsValid(new SemesterValidatorStrategy(Semester));
		if (validatorResult.IsValid)
		{
			await _viewModelService.RepositoryManager.SaveSemester(Semester);
			_viewModelService.Logger.LogInformation("Semester saved");
			await _viewModelService.NavigationManager.NavigateBack();
		}
		else
		{
			await _viewModelService.DialogService.ShowAlertAsync(validatorResult.ErrorMessage);
			_viewModelService.Logger.LogInformation("Validation error in semester form");
		}
	}
}