﻿using System.Collections.ObjectModel;
using CommunityToolkit.Mvvm.ComponentModel;
using CommunityToolkit.Mvvm.Input;
using Microsoft.Extensions.Logging;
using StudiApp.Interfaces;
using StudiApp.Models;
using StudiApp.Views;

namespace StudiApp.ViewModels;

public partial class SettingsViewModel : ObservableObject
{
	private readonly IViewModelService _viewModelService;

	[ObservableProperty] private ObservableCollection<Semester> _semesters;

	public SettingsViewModel(IViewModelService viewModelService)
	{
		_viewModelService = viewModelService;
	}

	[RelayCommand]
	public async Task OnAppearing()
	{
		await LoadAllSemesters();
	}

	public async Task LoadAllSemesters()
	{
		var semestersList = await _viewModelService.RepositoryManager.GetAllSemesters();
		Semesters = new ObservableCollection<Semester>(semestersList);
	}

	[RelayCommand]
	public async Task DeleteSemester(Semester semester)
	{
		var result = await _viewModelService.DialogService.ShowDeleteConfirmationAsync(nameof(Semester), semester.Name);
		if (result)
		{
			await _viewModelService.RepositoryManager.DeleteSemester(semester);
			await LoadAllSemesters();
			_viewModelService.Logger.LogInformation("Removed semester");
		}
	}

	[RelayCommand]
	public async Task NewSemester()
	{
		await _viewModelService.NavigationManager.NavigateTo(nameof(SemestersCreateView));
		_viewModelService.Logger.LogInformation($"Navigating to {nameof(SemestersCreateView)}");
	}

	[RelayCommand]
	public async Task EditSemester(Semester semester)
	{
		await _viewModelService.NavigationManager.NavigateTo(nameof(SemestersEditView), "Id", semester.Id);
		_viewModelService.Logger.LogInformation($"Navigating to {nameof(SemestersEditView)}");
	}
}